import asyncio
import logging

import uvloop
import aiohttp

class TrelloRest(object):

    def __init__(self):
        """
        TODO: dictate preference of receiving class-level variables
        api_token, api_key, api_url, api_organisation)
        """
        self._boards = dict()
        self._lists = dict()
        self._cards = dict()
        self._members = dict()
        self._api_url = ""
        self._api_token = ""
        self._api_key = ""
        self._organisation = ""
        asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
        self._loop = asyncio.get_event_loop()
        self._logger = logging.getLogger('trelloreporter')

    def configure_api_parameters(self, api_token, api_key, api_url, api_organisation):
        """
        A function to initialise the API parameters for Trello Method calls
        :param api_token: The API Token
        :param api_key: The API Key
        :param api_url: The API URL
        :param api_organisation: The API organisation
        :return: <None>
        """
        self._api_token = api_token
        self._api_key = api_key
        self._api_url = api_url
        self._organisation = api_organisation
        self._logger.info('Trello API configuration URL:{0} API Organisation:{1}'.format(api_url, api_organisation))


    #Accessor methods for the api_url, api_token, api_key, api_organisation
    #https://docs.python.org/3.5/library/asyncio-task.html - co-routines used to synchronise states
    @property
    def api_url(self):
        """
        Accessor for API URL
        :return: <string> return the value of the API URL
        """
        return self._api_url

    @api_url.setter
    def api_url(self, url_string):
        """

        :param url_string: Set the value of the API string
        """
        #TODO : validate strings - sanitize URL
        self._api_url = url_string

    @property
    def api_token(self):
        """
        Accessor for the api token
        :return:<string> return the current value of the API token
        """
        return self._api_token

    @api_token.setter
    def api_token(self, token_string):
        """
        Set the value of _api_token
        :param token_string: <string> value of the token
        """
        self._api_token = token_string

    @property
    def organisation(self):
        """

        :return: <string> Returns the _organisation object
        """
        return self._organisation

    @organisation.setter
    def organisation(self, org_string):
        """

        :param org_string: <string> the trello organisation to perform lookups against
        """
        #TODO validate strings or perform a lookup on trello ???
        self._organisation = org_string

    #Accessor/Mutator methods for the trello elements
    @property
    def boards(self):
        """

        :return: A dictionary of boards read from the trello API
        """
        return self._boards

    @boards.setter
    async def boards(self):
        """
        Read boards of the organisation using the self._api_url
        Chained to property methods:
            lists
            cards
            members
        """
        await self.populate_boards()

    @property
    def lists(self):
        """

        :return: <dict> A dictionary containing all the lists of the orgnanisation
        """
        return self._lists

    @lists.setter
    async def lists(self):
        """
        Method to populate all the trello lists of the self._organisation
        """
        if self._boards == {}:  #if the self._boards has not been populated, populate
            await self.populate_boards() #wait for the populate_boards()

        await self.populate_lists()

    @property
    def cards(self):
        """

        :return: <dict> returns a dict of the organisation's cards self._cards
        """
        return self._cards

    @cards.setter
    async def cards(self):
        """
        Fetch the cards of the self._organisation and populate them into self._cards
        """
        if self._boards == {}:
            await self.populate_boards()

        await self.populate_cards()

    @property
    def members(self):
        """

        :return: <dict> return a dict of the members of the self._organisation
        """

        return self._members

    @members.setter
    async def members(self):
        """
        Retrieve the members of the self._organisation from the Trello API
        """
        await self.populate_members()

    async def populate_boards(self):
        """
        Asynchronous method to populate the boards of the self._organisation, using the class-lvel
        self._api_url, self._api_token, self._api_key, self._organisation
        """
        self._logger.info('Attempting to populate board data from Trello')
        url_string = "{0}/organizations/{1}/boards?key={2}&token={3}".format(self._api_url,
                                                                            self._organisation,
                                                                            self._api_key,
                                                                            self._api_token
                                                                            )
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url_string) as response_string:
                    resp = await response_string.json()

        except Exception:
            self._logger.exception('Error connecting to Trello for board population')

        for board in range(len(resp)):
            board_text = resp[board]
            board_id = board_text['id']
            self._boards[board_id] = board_text
            self._logger.debug('Found board: {0}'.format(board_text['name']))

        self._logger.info('{0} boards populated successfully from Trello'.format(len(resp)))

    async def populate_lists(self):
        """
        Async method to facilitate the requesting of lists under a trello organisation
        """
        self._logger.info('Attempting to populate list data from Trello')
        if self._boards == {}:
            self._logger.info('Lists waiting for boards to populate')
            await self.populate_boards() #Ensure the boards are populated

        for board in self._boards.keys():
            try:
                url_string = "{0}/boards/{1}/lists?key={2}&token={3}".format(self._api_url,
                                                                                board,
                                                                                self._api_key,
                                                                                self._api_token
                                                                                )
                async with aiohttp.ClientSession() as session:
                    async with session.get(url_string) as response_string:
                        self._lists[board] = dict() #Initialise a dict at this boardID
                        resp = await response_string.json()
                        for list in resp:
                            list_id = list['id']
                            self._lists[board][list_id] = list
                            self._logger.debug('Found list: {0}'.format(list['name']))
            except Exception:
                self._logger.exception('Error connecting to Trello for list population')

        self._logger.info('Lists populated successfully from Trello')

    async def populate_cards(self):
        """
        Method to read the cards of an organisation from the trello API and store them as a dict
        using the board_id and the card_id as the keys
        """
        self._logger.info('Attempting to populate card data from Trello')
        if self.cards == {}:
            self._logger.info('Cards waiting for boards to populate')
            await self.populate_boards() #Ensure the boards are populated

        for board in self._boards.keys():
            try:
                url_string = "{0}/boards/{1}/cards/?key={2}&token={3}".format(self._api_url,
                                                                              board,
                                                                              self._api_key,
                                                                              self._api_token
                                                                              )
                async with aiohttp.ClientSession() as session:
                    async with session.get(url_string) as response_string:
                        resp = await response_string.json()
                        self._cards[board] = dict()
                        for card in resp:
                            card_id = card['id']
                            self._cards[board][card_id] = card
                            self._logger.debug('Found card: {0}'.format(card['desc']))

            except Exception:
                self._logger.exception('Error connecting to Trello for card population')

        self._logger.info('Cards populated successfully from Trello')

    async def populate_members(self):
        """
        Populate the members under the organisation, storing each member in the members dict()
        """
        self._logger.info('Attempting to populate member data from Trello')
        url_string = "{0}/organization/{1}/members?key={2}&token={3}".format(self._api_url,
                                                                             self._organisation,
                                                                             self._api_key,
                                                                             self._api_token)
        try:
            async with aiohttp.ClientSession() as session:
                response_string = await session.get(url_string)
                async with response_string:
                    resp = await response_string.json()
                    for member in range(len(resp)):
                        member_text = resp[member]
                        member_id = member_text['id']
                        self._members[member_id] = member_text
                        self._logger.debug('Found member: {0}'.format(member_text['fullName']))
                    self._logger.info(
                        '{0} members populated successfully from Trello'.format(len(resp)))
        except Exception:
            self._logger.exception('Error connecting to Trello for board member population')


import requests
from requests import Session

from trelloreporter.rest import trello

class TrelloReader(object):

    def __init__(self, apiToken, apiKey, apiURL, apiOrganisation):
        """

        :param apiToken: The API token
        :param apiKey:   The API key
        :param apiURL:   The API URL
        :param apiOrganisation: The API Organisation name as it appears in trello
        """
        self.api_token = apiToken
        self.api_key = apiKey
        self.api_url = apiURL
        self.boards = dict()
        self.cards = dict()
        self.lists = dict()
        self.members = dict()
        self.organisation = apiOrganisation

    def populate_boards(self):
        """
        Boards are the highest level concept within the Trello workflow.
        The Boards API allows you to list, view, create, and edit Boards.
        Each Board has a name, description, a set of members attached, and an ordered array of Lists.

        Populates the boards found under the api_key's endpoint as a JSON dict.
        :return: <null> initialises the cards to the json list of boards
        """
        urlString = "{0}/organizations/{1}/boards?key={2}&token={3}".format(self.api_url, self.organisation, self.api_key,
                                                                      self.api_token)
        responseString = requests.get(urlString)
        resp = responseString.json()

        for board in range(len(resp)):
            boardText = resp[board]
            boardID = boardText['id']
            self.boards[boardID] = boardText

    def populate_cards(self):
        """
        Extracts the cards found at the api_key's endpoint, storing each
        card as a JSON item in a dict
        :return:
        """
        for board in self.boards.keys():
            boardID = board
            urlString = "{0}/boards/{1}/cards/?key={2}&token={3}".format(self.api_url, boardID, self.api_key, self.api_token)
            responseString = requests.get(urlString)
            self.cards[boardID] = dict() #Initialise a new key using the ID extracted from the response
            for card in responseString.json():
                cardID = card['id']
                self.cards[boardID][cardID] = card

    def populate_lists(self):
        """
        Populates the lists found under the organisation and stores them in the lists() dictionary
        :return: <dict> of Trello lists
        """
        for board in self.boards.keys():
            boardID = board
            urlString = "{0}/boards/{1}/lists/?key={2}&token={3}".format(self.api_url, boardID, self.api_key, self.api_token)
            responseString = requests.get(urlString)
            self.lists[boardID] = dict() #Initialise a dictionary at lists.[boardID]
            for list in responseString.json():
                listID = list['id']
                self.lists[boardID][listID] = list

    def populate_members(self):
        """
        Method to populate the members of the Trello organisation
        :return: <null> : Initialises the members() dict
        """
        urlString = "{0}/organizations/{1}/members?key={2}&token={3}".format(self.api_url, self.organisation,
                                                                             self.api_key, self.api_token)

        responseString = requests.get(urlString)
        resp = responseString.json()

        for member in range(len(resp)):
            memberText = resp[member]
            memberID = memberText['id']
            self.members[memberID] = memberText

    def get_members(self):
        """

        :return: <dict> : return the members of the organisation
        """
        return self.members

    def get_list_cards(self,listID):
        """
        Returns a dict of cards based on a filter argument "lane"

        :param lane: <str> The lane in which the cards reside
        :return: <list> The cards which all belong to the lane
        """
        listFound = 0   #Marker for list found: this remains zero if there is no list which matches the search
        myCards = dict() #A place to store cards belonging to the list
        for boardID in self.boards.keys():
            for cardID in self.cards[boardID]:
                if listID == self.cards[boardID][cardID]['idList']:
                    myCards[cardID] = self.cards[boardID][cardID]
                    listFound = 1   #Set the marker to one

        if listFound == 1:
            return myCards
        else:
            return "List " + listID + " not found"

    def get_boards(self):
        """

        :return: <list> : returns a json array of boards
        """
        return self.boards

    def get_board(self, boardID):
        """
        :param: <str> : the ID of the board to return
        :return: <dict> : returns a json array of boards
        """
        if boardID in self.boards.keys():
            return self.boards[boardID]
        else:
            return "Board "+ boardID +" not found"

    def get_board_cards(self, boardID):
        """

        :param boardID: The boardID of the cards to return
        :return: <dict> All the cards which belong ot the board at hand
        """
        if boardID in self.boards.keys():
            return self.cards[boardID]
        else:
            return "Board " + boardID + " not found"

    def get_board_lists(self, boardID):
        """

        :param boardID: the boardID of the lists
        :return: <dict> : returns a dict of all the lists in a given board
        """
        if boardID in self.boards.keys():
            return self.lists[boardID]
        else:
            return "Board " + boardID + " not found"

    def get_lists(self):
        """

        :return: <dict> : returns a dict of lists
        """
        return self.lists

    def get_list(self, listID):
        """
        :param listID:  <str> : the ID of the list
        :return: <dict> : returns a json representation of a list item
        """
        for board in self.boards.keys():
            boardID = board
            for list in self.lists:
                if listID in self.lists[boardID].keys():
                    return self.lists[boardID][listID]

        return "List " + listID + " not found" #At the end of the loop, there is no list found

    def get_cards(self):
        """

        :return: <dict> : returns a json dict of cards
        """
        return self.cards

    def get_card(self, cardID):
        """
        :param cardID: <str> : the ID of the card to retrieve
        :return: <dict> : returns a json representation of the card
        """
        for board in self.boards.keys():
            boardID = board
            for card in self.cards:
                if cardID in self.cards[boardID].keys():
                    return self.cards[boardID][cardID]

        return "Card " + cardID + " not found" #At the end of the loop there is no card found

    def get_card_summary(self, cardID):
        """
        :parameter : str : The ID of the card to return a summary of
        :return: <json> A summarized version of the card object in cards[boardID][boards]
        """
        for board in self.boards.keys():
            boardID = board
            for card in self.cards:
                if cardID in self.cards[boardID].keys():
                    myCard = {'Board':'','Description':'','Status':'','DueDate':'','Members':'','LastUpdate':'','URL':''}
                    myCard['Board'] = self.boards[boardID]['name']
                    myCard['Description'] = self.cards[boardID][cardID]['name']
                    listID = self.cards[boardID][cardID]['idList']
                    myCard['Status'] = self.lists[boardID][listID]['name']
                    myCard['DueDate'] = self.cards[boardID][cardID]['due']
                    myCard['LastUpdate'] = self.cards[boardID][cardID]['dateLastActivity']
                    myCard['URL'] = self.cards[boardID][cardID]['url']
                    myCard['Members'] = dict()
                    for memberID in self.cards[boardID][cardID]['idMembers']:
                        memberUserName = self.members[memberID]['username']
                        myCard['Members'][memberUserName] = self.members[memberID]['fullName']
                    return myCard

        return "Card " + cardID + " not found"


import sys
import json
import time
import os
import re
from os.path import basename
from datetime import datetime
import logging
from logging import config
import smtplib
import sqlite3
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication
sys.path.append('../../')

from trelloreporter.writer import excel
from trelloreporter.config import yaml
from trelloreporter.rest import elastic

#Global variables: pass-by-reference messy
instance = excel.TrelloReader()
es_instance = elastic.ElasticRest()
logger = logging.getLogger('trelloreporter')
mail_config = dict()
connection = sqlite3.connect('/opt/trelloreporter/data/trelloreporter.db')
cursor = connection.cursor()
options = {'boards':0, 'lists':0, 'cards':0, 'pretty':0, 'spreadsheet':0, 'email':0, 'unsubscribe':0}


def setup_config(config_file_path):
    """
    A function to setup configuration paratmers.
    :param config_file_path: <str> : a file path to a configuration file, can be gathered
                                    from environment variable TRELLO_CONFIG or argv
    """
    config = yaml
    try:
        config_values = config.Yaml.read(os.getenv("TRELLO_CONFIG", default=config_file_path))
    except Exception as ex:
        print(ex)

    #Configure logging using the config file
    logging.config.dictConfig(config_values['log_config'])
    logger.info('=============Start=============')

    logger.info('Using config {0}'.format(config_file_path))

    instance.configure_api_parameters(
        api_token=os.getenv("TRELLO_TOKEN", default=config_values['api']['token']),
        api_key=os.getenv("TRELLO_KEY", default=config_values['api']['key']),
        api_url=os.getenv("TRELLO_URL", default=config_values['api']['url']),
        api_organisation=os.getenv("TRELLO_ORG", default=config_values['api']['organization'])
    )

    es_instance.configure_elastic_parameters(
        username=os.getenv("ES_USERNAME", default=config_values['elasticsearch']['username']),
        password=os.getenv("ES_PASSWORD", default=config_values['elasticsearch']['password']),
        endpoint=os.getenv("ES_ENDPOINT", default=config_values['elasticsearch']['endpoint']),
        port=os.getenv("ES_PORT", config_values['elasticsearch']['port']),
        index=os.getenv("ES_INDEX", default=config_values['elasticsearch']['index'])
    )

    mail_config['Subject'] = config_values['mail']['subject']
    mail_config['From'] = config_values['mail']['from']
    mail_config['Recipients'] = config_values['mail']['recipients']
    mail_config['Relay'] = config_values['mail']['relay']

    logger.info('Checking config file for new subscribers...')
    if mail_config['Recipients'] == [None]:
       logger.info('No subscribers in the config file')
       mail_config['Recipients'] = list()
    else:
       add_subscribers(mail_config['Recipients'])
    
    for recipient in connection.execute('SELECT * FROM subscribers').fetchall():
       mail_config['Recipients'].append(recipient)
    
    if options['email'] == 1:
       index = sys.argv.index('-e')
       email_address = str(sys.argv[index + 1])
       if mail_config['Recipients'] == [None]:
          mail_config['Recipients'] = email_address
       else:
          mail_config['Recipients'].append([email_address])
       add_subscribers([email_address])
     
    logger.info(json.dumps(mail_config, indent=4))
    logger.info('Mail configuration completed. Recipients: {0}'.format(mail_config['Recipients']))

def write_to_elastic():
    """
    Write card content to Elasticsearch
    :return:
    """
    if datetime.today().day == es_instance.search_latest_document(type="card").day:
        logger.info("Data found in Elastic for today: no updates to elastic")
    else:
        logger.info('No data found in Elasticsearch for today: writing documents')
        logger.debug('Boards {0}'.format(instance.get_boards().keys()))
        for board in instance.get_boards().keys():
            logger.debug('Writing card documents for board ID {0} to Elasticsearch'.format(board))
            for card in instance.get_board_cards(board):
                es_instance.write_to_elastic(instance.get_card_summary(card), 'card')

def generate_spreadsheet():
    """
    Write data to excel
    """
    instance.write_to_excel('/opt/trelloreporter/data/TrelloReport-'+ datetime.now().strftime('%Y-%m-%d') +'.xlsx')

def add_subscribers(subscribers):
    """

    :param subscribers: <list> a list of subscribers to add to the SQLite database
    :return:
    """
    cursor.execute('CREATE TABLE IF NOT EXISTS subscribers (email text)')

    for subscriber in subscribers:
        address = (subscriber,)
        cursor.execute('SELECT * FROM subscribers WHERE email=?', address)
        if cursor.fetchone() == None:
            cursor.execute('INSERT INTO subscribers VALUES (\'{0}\')'.format(subscriber))
            logger.info('No subscribers found in the database')
            logger.info('Adding user to subscribers: {0}'.format(subscriber))

    #Commit the changes
    connection.commit()

def remove_subscriber(subscriber):
    """
    Remove subscribers from the SQL database
    :param subscriber: <str> a subscriber to remove
    :return:
    """
    address = (subscriber,)
    if cursor.execute('SELECT * FROM subscribers WHERE email=(?)', address).fetchone() == None:
        logger.error('No subscriber found {0}'.format(subscriber))
    else:
        cursor.execute('DELETE FROM subscribers WHERE email=(?)', address)
        logger.info('Subscriber deleted: {0}'.format(subscriber))

    connection.commit()

def dispatch_spreadsheet():
    """
    A method to send an email of the spreadsheet (as an attachment)
    :return:
    """
    #Create a text/plain message, populate mail headers
    message = MIMEMultipart()
    message['Subject'] = mail_config['Subject']
    message['From'] = mail_config['From']

    message.attach(MIMEText('{0} TrelloReport attached'.format(datetime.today())))

    filename = '/opt/trelloreporter/data/TrelloReport-{0}.xlsx'.format(datetime.now().strftime('%Y-%m-%d'))
    with open(filename, 'rb') as file:
        part = MIMEApplication(file.read(), Name=basename(filename))

    part['Content-Disposition'] = 'attachment; filename={0}'.format(basename(filename))
    logger.info('Attaching file: {}'.format(filename))
    message.attach(part)

    logger.info('Preparing to dispatch spreadsheet using SMTP')
    try:
        smtp_service = smtplib.SMTP(mail_config['Relay'])
        smtp_service.sendmail(from_addr=mail_config['From'],to_addrs=mail_config['Recipients'],
                          msg=message.as_string())
        smtp_service.quit()
    except Exception as ex:
        logger.info('Failed to send email... {0}'.format(ex))

def main():
    start_time = time.time()
    # A static configuration file
    config_file_path = "/opt/trelloreporter/config/trelloreporter.yml"

    usage = "trelloreporter \n\n " \
            "Usage: trelloreporter [option] --config <config.yml>\n" \
            " config.Yaml is the Yaml configuration file\n" \
            " Options:\n" \
            "   -b            Print boards\n" \
            "   -l            Print lists\n" \
            "   -c            Print cards\n" \
            "   -s            Generate an excel spreadsheet\n" \
            "   -e            Subscribe to the TrelloReporter via email\n" \
            "   -r            Remove a subscriber from the database\n" \
            "   --pretty      Pretty-Print the json data\n" \
            "   --config      Specify a Yaml file containing the API credentials\n\n" \
            "Example: \n" \
            "   trelloreporter -b --pretty --config myconfigfile.yml"

    if len(sys.argv) == 1:
       print(usage)
       sys.exit(0)

    #Process the command line arguments
    #Check for config file
    try:
        config_index = sys.argv.index("--config")
        config_file_path = sys.argv[config_index + 1]
        setup_config(config_file_path=config_file_path)

    except ValueError:
        print("Assuming default configuration")
        print("==========")
        print(usage)
        print("==========")
        setup_config(config_file_path=config_file_path)

    #Start with the display output filters
    # If no index is found, the list.index() method throws a ValueError
    try:
        display_index = sys.argv.index('-b')
        options['boards'] = 1
    except Exception as ex:
        logger.info('Board display not selected')

    try:
        display_index = sys.argv.index('-l')
        options['lists'] = 1
    except ValueError:
        logger.info('List display not selected')

    try:
        display_index = sys.argv.index('-c')
        options['cards'] = 1
    except:
        logger.info('Card display not selected')

    try:
        display_index = sys.argv.index('-s')
        options['spreadsheet'] = 1
    except:
        logger.info('Spreadsheet not selected')

    try:
        display_index = sys.argv.index('-e')
        options['email'] = 1
    except:
        logger.info('Email subscription dispatch not selected')

    try:
        pretty_index = sys.argv.index("--pretty")
        options['pretty'] = 1
    except ValueError:
        logger.info("Dumping raw JSON: pretty-printing not set")

    try:
        display_index = sys.argv.index('-r')
        options['unsubscribe'] = 1
        try:
           remove_subscriber(sys.argv[display_index + 1])
        except:
           logger.error('Unable to read supplied email address... was it provided?')
    except ValueError:
        logger.info('Unable to read supplied email address')


    logger.debug('Reading member data from the trello API')
    instance.populate_members()
    logger.debug('Reading list data from the trello API')
    instance.populate_lists()
    logger.debug('Reading trello cards from the trello API')
    instance.populate_cards()

    if options['pretty'] == 1:
        if options['boards'] == 1:
            print(json.dumps(instance.get_boards(), indent=4, sort_keys=True))
            print("==========")
        if options['lists'] == 1:
            print(json.dumps(instance.get_lists(), indent=4, sort_keys=True))
            print("==========")
        if options['cards'] == 1:
            for board_id in instance.get_cards().keys():
                for card in instance.get_board_cards(board_id):
                    print(json.dumps(instance.get_card_summary(card), indent=4, sort_keys=True))
            print("==========")
    else:
        if options['boards'] == 1:
            print(instance.get_boards())
            print("==========")
        if options['lists'] == 1:
            print(instance.get_lists())
            print("==========")
        if options['cards'] == 1:
            print(instance.get_cards())
            print("==========")

    #Push data into Elasticsearch
    write_to_elastic()

    #Generate spreadsheet on the first day of the month or when the generate spreadsheet switch is set.
    if datetime.today().day == 1 or options['spreadsheet'] == 1:
        generate_spreadsheet()
        dispatch_spreadsheet()

    connection.close()

    logger.info('Runtime: {:.2f}s'.format(time.time() - start_time))
    logger.info('=============End=============')
    exit()

main()



import asyncio
import logging

import uvloop
from openpyxl import Workbook
from openpyxl.styles import NamedStyle
from openpyxl.styles.cell_style import CellStyle

from trelloreporter.rest import trello

class TrelloReader(object):

    def __init__(self):
        """

        """
        self._reader = trello.TrelloRest()
        self._cards = dict()
        self._lists = dict()
        self._members = dict()
        self._boards = dict()

        asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
        self._loop = asyncio.get_event_loop()

        self._logger = logging.getLogger('trelloreporter')

    def configure_api_parameters(self, api_token, api_key, api_url, api_organisation):
        """
        A function to initialise the API parameters of the trello reader object
        :param api_token: the API token
        :param api_key: the API key
        :param api_url: the API URL
        :param api_organisation: the API organisation
        :return: <none>
        """
        self._reader.configure_api_parameters(api_token, api_key, api_url, api_organisation)

    def populate_boards(self):
        """
        Boards are the highest level concept within the Trello workflow.
        The Boards API allows you to list, view, create, and edit Boards.
        Each Board has a name, description, a set of members attached, and an ordered array of Lists.

        Populates the boards found under the api_key's endpoint as a JSON dict.
        :return: <null> initialises the cards to the json list of boards
        """
        self._loop.run_until_complete(self._reader.populate_boards())
        self._boards = self._reader.boards

    def populate_cards(self):
        """
        Extracts the cards found at the api_key's endpoint, storing each
        card as a JSON item in a dict
        :return:
        """
        if self._boards == {}:
            self.populate_boards()
        self._loop.run_until_complete(self._reader.populate_cards())
        self._cards = self._reader.cards

    def populate_lists(self):
        """
        Populates the lists found under the organisation and stores them in the lists() dictionary
        :return: <dict> of Trello lists
        """
        if self._boards == {}:
            self.populate_boards()
        self._loop.run_until_complete(self._reader.populate_lists())
        self._lists = self._reader.lists

    def populate_members(self):
        """
        Method to populate the members of the Trello organisation
        :return: <null> : Initialises the members() dict
        """
        self._loop.run_until_complete(self._reader.populate_members())
        self._members = self._reader.members

    def get_members(self):
        """

        :return: <dict> : return the members of the organisation
        """
        return self._members

    def get_list_cards(self, list_id):
        """
        Returns a dict of cards based on a filter argument "lane"

        :param lane: <str> The lane in which the cards reside
        :return: <list> The cards which all belong to the lane
        """
        list_found = 0   #Marker for list found: this remains zero if there is no list which matches the search
        my_cards = dict() #A place to store cards belonging to the list
        for board_id in self._boards.keys():
            for card_id in self._cards[board_id]:
                if list_id == self._cards[board_id][card_id]['idList']:
                    my_cards[card_id] = self._cards[board_id][card_id]
                    list_found = 1   #Set the marker to one

        if list_found == 1:
            return my_cards
        else:
            return "List " + list_id + " not found"

    def get_boards(self):
        """

        :return: <list> : returns a json array of boards
        """
        return self._boards

    def get_board(self, board_id):
        """
        :param: <str> : the ID of the board to return
        :return: <dict> : returns a json array of boards
        """
        if board_id in self._boards.keys():
            return self._boards[board_id]
        else:
            return "Board " + board_id + " not found"

    def get_board_cards(self, board_id):
        """

        :param board_id: The boardID of the cards to return
        :return: <dict> All the cards which belong ot the board at hand
        """
        if board_id in self._boards.keys():
            return self._cards[board_id]
        else:
            return "Board " + board_id + " not found"

    def get_board_lists(self, board_id):
        """

        :param board_id: the boardID of the lists
        :return: <dict> : returns a dict of all the lists in a given board
        """
        if board_id in self._boards.keys():
            return self._lists[board_id]
        else:
            return "Board " + board_id + " not found"

    def get_lists(self):
        """

        :return: <dict> : returns a dict of lists
        """
        return self._lists

    def get_list(self, list_id):
        """
        :param list_id:  <str> : the ID of the list
        :return: <dict> : returns a json representation of a list item
        """
        for board in self._boards.keys():
            board_id = board
            for list in self._lists:
                if list_id in self._lists[board_id].keys():
                    return self._lists[board_id][list_id]

        return "List " + list_id + " not found" #At the end of the loop, there is no list found

    def get_cards(self):
        """

        :return: <dict> : returns a json dict of cards
        """
        return self._cards

    def get_card(self, card_id):
        """
        :param card_id: <str> : the ID of the card to retrieve
        :return: <dict> : returns a json representation of the card
        """
        for board in self._boards.keys():
            board_id = board
            for card in self._cards:
                if card_id in self._cards[board_id].keys():
                    return self._cards[board_id][card_id]

        return "Card " + card_id + " not found" #At the end of the loop there is no card found

    def get_card_summary(self, card_id):
        """
        :parameter : str : The ID of the card to return a summary of
        :return: <json> A summarized version of the card object in cards[board_id][boards]
        """
        for board in self._boards.keys():
            board_id = board
            for card in self._cards:
                if card_id in self._cards[board_id].keys():
                    self._logger.debug('Summarizing card ID: {0} {1}'.format(
                        card_id, self._cards[board_id][card_id]['name']))
                    my_card = {'Board':'',
                               'Description':'',
                               'Status':'',
                               'DueDate':'',
                               'Members':'',
                               'LastUpdate':'',
                               'URL':''}
                    my_card['Board'] = self._boards[board_id]['name']
                    my_card['Description'] = self._cards[board_id][card_id]['name']
                    listID = self._cards[board_id][card_id]['idList']
                    my_card['Status'] = self._lists[board_id][listID]['name']
                    my_card['DueDate'] = self._cards[board_id][card_id]['due']
                    my_card['LastUpdate'] = self._cards[board_id][card_id]['dateLastActivity']
                    my_card['URL'] = self._cards[board_id][card_id]['url']
                    my_card['Members'] = dict()
                    counter = 1
                    for member_id in self._cards[board_id][card_id]['idMembers']:
                        try:
                            member_user_name = self._members[member_id]['username']
                            my_card['Members'][member_user_name] = self._members[member_id]['fullName']
                        except KeyError:
                            my_card['Members']['Removed Member ID {0}:'.format(counter)] = member_id
                            counter = counter + 1
                    return my_card

        return "Card " + card_id + " not found"

    def get_board_summary(self, board_id):
        """
        :parameter : str : The ID of the card to return a summary of
        :return: <json> A summarized version of the card object in cards[board_id][boards]
        """
        card_id = ""
        for card_id in self._cards[board_id].keys():
            self._logger.debug('Summarizing card ID: {0} {1}'.format(
                card_id, self._cards[board_id][card_id]['name']))
            my_card = {'Board':'',
                       'Description':'',
                       'Status':'',
                       'DueDate':'',
                       'Members':'',
                       'LastUpdate':'',
                       'URL':''}
            my_card['Board'] = self._boards[board_id]['name']
            my_card['Description'] = self._cards[board_id][card_id]['name']
            listID = self._cards[board_id][card_id]['idList']
            my_card['Status'] = self._lists[board_id][listID]['name']
            my_card['DueDate'] = self._cards[board_id][card_id]['due']
            my_card['LastUpdate'] = self._cards[board_id][card_id]['dateLastActivity']
            my_card['URL'] = self._cards[board_id][card_id]['url']
            my_card['Members'] = dict()
            counter = 1
            for member_id in self._cards[board_id][card_id]['idMembers']:
                try:
                    member_user_name = self._members[member_id]['username']
                    my_card['Members'][member_user_name] = self._members[member_id]['fullName']
                except KeyError:
                    my_card['Members']['Removed Member ID {0}:'.format(counter)] = member_id
                    counter = counter + 1
            return my_card

        return "Card " + card_id + " not found"
    def write_to_excel(self, filename):
        """
         Create named styles, using the bellow color schemes in Excel
         Neutral, Bad, Input, WarningText, Good
        """
        self._logger.info('Creating spreadsheet')
        wb = Workbook()
        ws = wb.active  # get the active workbook

        for board in self.get_boards().keys():
            # Name the worksheet using the board name
            board_object = self.get_board(board_id=board)
            board_name = board_object['name'].replace('[', '(')  # Remove invalid characters

            # Ensure the sheet name is < 31 characters: Outlook fails to open anything else
            trunc_board_name = (board_name[:31]) if len(board_name) > 31  else board_name

            ws.title = trunc_board_name.replace(']', ')')
            self._logger.debug('Create worksheet :{0}'.format(ws.title))

            # Set the column headings
            ws['A1'] = "Task Name"
            ws['B1'] = "Status"
            ws['C1'] = "Due Date"
            ws['D1'] = "Members"
            ws['E1'] = "URL"
            row = 2
            row_counter = 0  # N rows
            for card in self.get_board_cards(board):
                card_object = self.get_card_summary(card_id=card)
                ws['A{0}'.format(row)] = card_object['Description']
                ws['B{0}'.format(row)] = card_object['Status']
                ws['C{0}'.format(row)] = card_object['DueDate']

                if len(card_object['Members']) == 0:
                    ws['D{0}'.format(row)] = "None"
                else:
                    members = ""
                    for member in card_object['Members'].values():
                        members = member + '\n' + members
                    ws['D{0}'.format(row)] = members

                ws['E{0}'.format(row)] = card_object['URL']

                # Apply Styling for column B data i.e. Status
                if card_object['Status'] == 'Backlog':
                    cell = ws['C{0}'.format(row)]   #Column C: Due Date
                    cell.style = 'Accent4'
                elif card_object['Status'] == 'In Progress':
                    cell = ws['C{0}'.format(row)]
                    cell.style = 'Accent1'
                elif card_object['Status'] == 'Impeded':
                    cell = ws['C{0}'.format(row)]
                    cell.style = 'Bad'
                elif card_object['Status'] == 'Stopped':
                    cell = ws['C{0}'.format(row)]
                    cell.style = 'Accent2'
                elif card_object['Status'] == 'Completed':
                    cell = ws['C{0}'.format(row)]
                    cell.style = 'Good'

                # Apply styling to column E - URL
                ws['E{0}'.format(row)].style = 'Hyperlink'

                    # Apply colouring to the tab
                if 'Platform' in board_name:
                    ws.sheet_properties.tabColor = "976360"
                elif 'Client' in board_name:
                    ws.sheet_properties.tabColor = "6878CC"
                elif 'Core' in board_name:
                    ws.sheet_properties.tabColor = "FFA500"
                else:
                    ws.sheet_properties.tabColor = "500FFA"

                row_counter += row_counter

            # Sort by list name. Order:
            # * Backlog, In Progress, Impeded, Stopped, Incidents(?), Completed
            #ws.auto_filter.ref = 'A1:E{0}'.format(row_counter)
            #ws.auto_filter.add_filter_column(0, ["Backlog", "In Progress",
            #                                     "Impeded", "Stopped",
            #                                     "Completed"])
            #ws.auto_filter.add_sort_condition('B2:B{0}'.format(row_counter))  # Status column

            wb.create_sheet("New Worksheet")  # Generate a new worksheet
            ws = wb["New Worksheet"]

        try:
            wb.save(filename)
            self._logger.info('Spreadsheet {0} successfully created'.format(filename))
        except Exception:
            self._logger.error('Unable to save {0}'.format(filename))


#!/usr/bin/env python

# Copyright 2017 Internet Solutions (Pty) Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import sys

from codecs import open

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
for p in sys.path:
    print(p)

local = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(local, 'trelloreporter', '__version__.py'), 'r', 'utf-8') as f:
    exec(f.read(), about)

packages = ['trelloreporter', 'trelloreporter/cmd', 'trelloreporter/config', 'trelloreporter/rest',
    'trelloreporter/trelloreader', 'trelloreporter/writer']

requires = [
    'requests>=2.12.5',
    'openpyxl>=2.4.5',
    'ruamel.yaml>=0.13.10',
    'uvloop>=0.8.1',
    'aiohttp>=2.3.2',
    'elasticsearch>=6.0.0'
]

test_requirements = [
    'coverage==4.4.1',
    'pytest==3.2.1',
    'pytest-cov==2.5.1',
]

setup(
    name='trelloreporter',
    version=about['__version__'],
    description='Platform Engineering Northbound API',
    author='Paul Stevens',
    author_email='paul.stevens@is.co.za',
    url='https://platform.is',
    packages=packages,
    package_dir={'trelloreporter': 'trelloreporter'},
    data_files=[('/opt/trelloreporter/config', ['trelloreporter/config/trelloreporter.yml']), 
	('/opt/trelloreporter/data', ['trelloreporter/data/trelloreporter.db']),
	('/opt/trelloreporter/data/log',['trelloreporter/data/log/trelloreporter.log'])],
    include_package_data=True,
    install_requires=requires,
    python_requires='~=3.5',
    license='Apache 2.0',
    zip_safe=False,
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ),
    cmdclass={},
    entry_points={
        'console_scripts': [
            'trelloreporter=trelloreporter.cmd.trelloreport:main',
        ]
    },
    tests_require=test_requirements,
    extras_require={},
)



import asyncio
from datetime import datetime
import logging

import uvloop
from elasticsearch import Elasticsearch

class ElasticRest(object):

    def __init__(self):
        """
        Configure the credentials required to access the ES endpoint
        """
        self._username = ""
        self._password = ""
        self._elastic_endpoint = ""
        self._elastic_port = ""
        self._elastic_index = ""

        asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
        self._loop = asyncio.get_event_loop()
        self._es_instance = Elasticsearch()
        self._logger = logging.getLogger('trelloreporter')

    def configure_elastic_parameters(self, username, password, endpoint, port, index):
        """
        A function to initialise the Elasticsearch parameters; ready for shipping data
        :param username: the ES username
        :param password: the ES password
        :param endpoint: the ES endpoint
        :param port: the ES port
        :param index: the index within ES
        :return: <None>
        """
        self._elastic_index = index
        self._elastic_port = port
        self._elastic_endpoint = endpoint
        self._password = password
        self._username = username
        self._es_instance = Elasticsearch([endpoint],
                                          http_auth=(username, password),
                                          port=port
                                          )

        self._logger.info(
            'Elasticsearch configuration endpoint:{0} port:{1} index:{2}'.format(endpoint, port, index)
            )

    def write_to_elastic(self, document, type):
        """
        curl -u elastic -XPUT "http://127.0.0.1:9200/trello/members/1/_create {document}"
        A function to write data to an ElasticSearch endpoint. The function adds a timestamp
        indicating the current time - this is for generating time-series data in ES
        :param document: The object to be written as an elastic document
        :param type: The type of document
        :return: 0 if the write was successful.
        """
        document['writeTime'] = datetime.now().isoformat()

        try:
            self._logger.debug('Attempting to create {0} document: {1}'.format(
                type, document['Description']))
            res = self._es_instance.index(index=self._elastic_index,
                                          doc_type=type,
                                          body=document)
            if (res['created']) is not 'True':
                self._logger.debug('{0} document written successfully'.format(
                    document['Description']))
                return 0
            else:
                self._logger.debug('{0} document not written'.format(
                    document['Description']))
                return 1

        except Exception as ex:
            self._logger.error('Writing to Elasticsearch failed: {0}'.format(ex))
            return 1

    def search_latest_document(self, type):
        """
        Return the timestamp of the most recent document in the index.
        The function uses 'writeTime' as the aggregation field.
        :param index: <str> : the index
        :param type: <str> : the type of document
        :return: <datetime> : a timestamp of the most recent document in the index

        Elastic REST call : GET http://<ES_HOST>:<ES_PORT>/trello_reports-/card/_search?q=writeTime:<date>
        """
        time_of_insert = datetime.now()
        try:
            self._logger.debug('Searching for latest {0} document'.format(type))
            result = self._es_instance.search(index=self._elastic_index,
                                              doc_type=type,
                                              sort="writeTime:desc",
                                              size=1)
            time_of_insert = datetime.strptime(result['hits']['hits'][0]['_source']['writeTime'],
                                               "%Y-%m-%dT%H:%M:%S.%f")
            self._logger.info('Latest time of insert for {0} is {1}'.format(type, time_of_insert))
        except Exception as ex:
            self._logger.error('Unable to obtain latest document in index {0}. ERROR :{1}'.format(
                self._elastic_index, ex))

        return time_of_insert
